// ported from django
module.exports = {
    name: 'ConditionalGet',
    make: function(settings, conf) {
        this.add('writeHead', function(statusCode, headers) {
            var share = this.share;
            if (share.omitMe === true) {
                this.next(statusCode, headers);
                return;
            };
            
            if (statusCode == 304) {
                // 304 already setted by upstream filter/handler, go next to avoid conflict
                this.next(statusCode, headers);
                return;
            }
            var etag = headers['etag'],
            lastModified = headers['last-modified'];
            if (etag && etag == share.noneMatch) {
                statusCode = 304;
            } else if (lastModified && lastModified == share.modifiedSince) {
                statusCode = 304;
            }
            if (statusCode == 304) {
                // 304 is setted by current filter, write to the client directly, response end.
                delete headers['content-length'];
                this.response.writeHead(statusCode, headers);
                this.response.end();
                return;
            }
            this.next(statusCode, headers);
        });

        return function(req, res, go) {
            var noneMatch = req.headers['if-none-match'],
            modifiedSince = req.headers['if-modified-since'];
            var share = this.share;
            if (!(req.method === "GET" && (modifiedSince || noneMatch))) {
                // tell my flake defined above, that it shouldn't be called in this session
                share.omitMe = true;
            } else {
                // inject request specific informations into context,
                // so that our prepared function chains can get the correct info.
                share.noneMatch = noneMatch;
                share.modifiedSince = modifiedSince;
            }
            go();
        }
    }
}